<?php

namespace Runtime\Http\Views;


use Runtime\Exceptions\ExceptionHandler;
use Twig\Environment;
use Twig\Error\LoaderError;
use Twig\Error\RuntimeError;
use Twig\Error\SyntaxError;
use Twig\Loader\FilesystemLoader;

class Factory implements \Runtime\Contracts\View\Factory {

    /**
     * @var string
     */
    private $name;

    /**
     * @var array
     */
    private $arguments;

    /**
     * @var \Twig\Environment
     */
    private $twig;

    /**
     * Factory constructor.
     */
    public function __construct()
    {
        $loader = new FilesystemLoader(app()->resourcePath()  . 'views');
        $this->twig = new Environment($loader, [
            'cache' => app()->resourcePath() . 'cache'
        ]);
    }

    /**
     * @param $name
     * @param array $arguments
     */
    public function make($name, $arguments = [])
    {
        $this->name = $this->name($name);
        $this->arguments = $arguments;

        try {
            $this->render();
        }
        catch (\Exception $e) {
            ExceptionHandler::make($e);
        }
    }

    /**
     * @param $name
     * @param $arguments
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    private function render()
    {
        echo $this->twig->render($this->name . '.html', $this->arguments);
    }

    /**
     * @param $name
     * @return string|string[]
     */
    private function name($name)
    {
        return str_replace('.', '/', $name);
    }
}